@@ -11,7 +11,6 @@ from toolz import accumulate, sliding_window
 from .. import sharedict
 from ..base import tokenize
 from ..compatibility import Sequence
-from ..utils import ignoring
 from . import chunk
 from .core import (Array, asarray, normalize_chunks,
                    stack, concatenate, block,
@@ -604,37 +603,26 @@ def tril(m, k=0):
     return Array(dsk, name, shape=m.shape, chunks=m.chunks, dtype=m.dtype)
 
 
-def offset_func(func, offset, *args):
-    """  Offsets inputs by offset
-
-    >>> double = lambda x: x * 2
-    >>> f = offset_func(double, (10,))
-    >>> f(1)
-    22
-    >>> f(300)
-    620
-    """
-    def _offset(*args):
+def _np_fromfunction(func, shape, dtype, offset, func_kwargs):
+    def offset_func(*args, **kwargs):
         args2 = list(map(add, args, offset))
-        return func(*args2)
+        return func(*args2, **kwargs)
 
-    with ignoring(Exception):
-        _offset.__name__ = 'offset_' + func.__name__
-
-    return _offset
+    return np.fromfunction(offset_func, shape, dtype=dtype, **func_kwargs)
 
 
 @wraps(np.fromfunction)
-def fromfunction(func, chunks=None, shape=None, dtype=None):
+def fromfunction(func, chunks=None, shape=None, dtype=None, **kwargs):
     if chunks:
         chunks = normalize_chunks(chunks, shape)
-    name = 'fromfunction-' + tokenize(func, chunks, shape, dtype)
+    name = 'fromfunction-' + tokenize(func, chunks, shape, dtype, kwargs)
     keys = list(product([name], *[range(len(bd)) for bd in chunks]))
     aggdims = [list(accumulate(add, (0,) + bd[:-1])) for bd in chunks]
     offsets = list(product(*aggdims))
     shapes = list(product(*chunks))
+    dtype = dtype or float
 
-    values = [(np.fromfunction, offset_func(func, offset), shp)
+    values = [(_np_fromfunction, func, shp, dtype, offset, kwargs)
               for offset, shp in zip(offsets, shapes)]
 
     dsk = dict(zip(keys, values))
