@@ -10,6 +10,7 @@ from ..core import flatten, reverse_dict
 from ..optimization import cull, fuse, inline_functions
 from ..utils import ensure_dict
 
+from numbers import Integral
 
 # All get* functions the optimizations know about
 GETTERS = (getter, getter_nofancy, getter_inline, getitem)
@@ -184,7 +185,7 @@ def check_for_nonfusible_fancy_indexing(fancy, normal):
     # x[:, [1, 2], :][0, :, :] -> x[0, [1, 2], :] or
     # x[0, :, :][:, [1, 2], :] -> x[0, [1, 2], :]
     for f, n in zip_longest(fancy, normal, fillvalue=slice(None)):
-        if type(f) is not list and isinstance(n, int):
+        if type(f) is not list and isinstance(n, Integral):
             raise NotImplementedError("Can't handle normal indexing with "
                                       "integers and fancy indexing if the "
                                       "integers and fancy indices don't "
@@ -229,7 +230,7 @@ def fuse_slice(a, b):
     if isinstance(b, slice):
         b = normalize_slice(b)
 
-    if isinstance(a, slice) and isinstance(b, int):
+    if isinstance(a, slice) and isinstance(b, Integral):
         if b < 0:
             raise NotImplementedError()
         return a.start + b * a.step
@@ -252,7 +253,7 @@ def fuse_slice(a, b):
 
     if isinstance(b, list):
         return [fuse_slice(a, bb) for bb in b]
-    if isinstance(a, list) and isinstance(b, (int, slice)):
+    if isinstance(a, list) and isinstance(b, (Integral, slice)):
         return a[b]
 
     if isinstance(a, tuple) and not isinstance(b, tuple):
@@ -276,7 +277,7 @@ def fuse_slice(a, b):
         result = list()
         for i in range(len(a)):
             #  axis ceased to exist  or we're out of b
-            if isinstance(a[i], int) or j == len(b):
+            if isinstance(a[i], Integral) or j == len(b):
                 result.append(a[i])
                 continue
             while b[j] is None:  # insert any Nones on the rhs
