@@ -758,16 +758,19 @@ class Bag(DaskMethodsMixin):
             return self.reduction(reduce(binop), reduce(combine),
                                   split_every=split_every)
 
-    def frequencies(self, split_every=None):
+    def frequencies(self, split_every=None, sort=False):
         """ Count number of occurrences of each distinct element.
 
         >>> b = from_sequence(['Alice', 'Bob', 'Alice'])
         >>> dict(b.frequencies())  # doctest: +SKIP
         {'Alice': 2, 'Bob', 1}
         """
-        return self.reduction(frequencies, merge_frequencies,
-                              out_type=Bag, split_every=split_every,
-                              name='frequencies').map_partitions(dictitems)
+        result = self.reduction(frequencies, merge_frequencies,
+                                out_type=Bag, split_every=split_every,
+                                name='frequencies').map_partitions(dictitems)
+        if sort:
+            result = result.map_partitions(sorted, key=second, reverse=True)
+        return result
 
     def topk(self, k, key=None, split_every=None):
         """ K largest elements in collection
@@ -1967,7 +1970,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
     max_branch = max_branch or 32
     n = b.npartitions
 
-    stages = int(math.ceil(math.log(n) / math.log(max_branch)))
+    stages = int(math.ceil(math.log(n) / math.log(max_branch))) or 1
     if stages > 1:
         k = int(math.ceil(n ** (1 / stages)))
     else:
