@@ -187,12 +187,13 @@ def test_read_glob(tmpdir, write_engine, read_engine):
 
     # Infer divisions for engines/versions that support it
 
-    ddf2 = dd.read_parquet(os.path.join(fn, '*'), engine=read_engine,
+    ddf2 = dd.read_parquet(os.path.join(fn, '*.parquet'), engine=read_engine,
                            infer_divisions=should_check_divs(write_engine) and should_check_divs(read_engine))
     assert_eq(ddf, ddf2, check_divisions=should_check_divs(write_engine) and should_check_divs(read_engine))
 
     # No divisions
-    ddf2_no_divs = dd.read_parquet(os.path.join(fn, '*'), engine=read_engine, infer_divisions=False)
+    ddf2_no_divs = dd.read_parquet(os.path.join(fn, '*.parquet'),
+                                   engine=read_engine, infer_divisions=False)
     assert_eq(ddf.clear_divisions(), ddf2_no_divs, check_divisions=True)
 
 
@@ -263,6 +264,20 @@ def test_columns_index(tmpdir, write_engine, read_engine):
               ddf.clear_divisions(), check_divisions=True)
 
 
+@write_read_engines_xfail
+def test_nonsense_column(tmpdir, write_engine, read_engine):
+    fn = str(tmpdir)
+    ddf.to_parquet(fn, engine=write_engine)
+    with pytest.raises((ValueError, KeyError)):
+        # fastparquet fails early, pyarrow only on compute
+        dd.read_parquet(fn, columns=['nonesense'], engine=read_engine
+                        ).compute()
+    with pytest.raises((Exception, KeyError)):
+        # fastparquet fails early, pyarrow only on compute
+        dd.read_parquet(fn, columns=['nonesense'] + list(ddf.columns),
+                        engine=read_engine).compute()
+
+
 @write_read_engines_xfail
 def test_columns_no_index(tmpdir, write_engine, read_engine):
     fn = str(tmpdir)
@@ -427,7 +442,8 @@ def test_optimize(tmpdir, c):
     assert_eq(df[c], ddf2[c])
     x = ddf2[c]
 
-    dsk = x.__dask_optimize__(x.dask, x.__dask_keys__())
+    with dask.config.set(fuse_rename_keys=False):
+        dsk = x.__dask_optimize__(x.dask, x.__dask_keys__())
     assert len(dsk) == x.npartitions
     assert all(v[4] == c for v in dsk.values())
 
