@@ -21,7 +21,7 @@ except ImportError:
 from ..compatibility import PY2, Iterator
 from ..core import get_deps
 from ..local import get_sync
-from ..utils import asciitable, is_arraylike
+from ..utils import asciitable, is_arraylike, Dispatch
 
 
 PANDAS_VERSION = LooseVersion(pd.__version__)
@@ -237,7 +237,21 @@ def _empty_series(name, dtype, index=None):
     return pd.Series([], dtype=dtype, name=name, index=index)
 
 
-def make_meta(x, index=None):
+make_meta = Dispatch('make_meta')
+
+
+@make_meta.register((pd.Series, pd.DataFrame))
+def make_meta_pandas(x):
+    return x.iloc[:0]
+
+
+@make_meta.register(pd.Index)
+def make_meta_index(x):
+    return x[:0]
+
+
+@make_meta.register(object)
+def make_meta_object(x, index=None):
     """Create an empty pandas object containing the desired metadata.
 
     Parameters
@@ -265,12 +279,9 @@ def make_meta(x, index=None):
     """
     if hasattr(x, '_meta'):
         return x._meta
-    if isinstance(x, (pd.Series, pd.DataFrame)):
-        return x.iloc[0:0]
-    elif isinstance(x, pd.Index):
-        return x[0:0]
     elif is_arraylike(x):
         return x[:0]
+
     index = index if index is None else index[0:0]
 
     if isinstance(x, dict):
@@ -307,6 +318,35 @@ else:
     _numeric_index_types = (pd.Int64Index, pd.Float64Index)
 
 
+meta_nonempty = Dispatch('meta_nonempty')
+
+
+@meta_nonempty.register(object)
+def meta_nonempty_object(x):
+    """Create a nonempty pandas object from the given metadata.
+
+    Returns a pandas DataFrame, Series, or Index that contains two rows
+    of fake data.
+    """
+    if is_scalar(x):
+        return _nonempty_scalar(x)
+    else:
+        raise TypeError("Expected Index, Series, DataFrame, or scalar, "
+                        "got {0}".format(type(x).__name__))
+
+
+@meta_nonempty.register(pd.DataFrame)
+def meta_nonempty_dataframe(x):
+    idx = meta_nonempty(x.index)
+    data = {i: _nonempty_series(x.iloc[:, i], idx=idx)
+            for i, c in enumerate(x.columns)}
+    res = pd.DataFrame(data, index=idx,
+                       columns=np.arange(len(x.columns)))
+    res.columns = x.columns
+    return res
+
+
+@meta_nonempty.register(pd.Index)
 def _nonempty_index(idx):
     typ = type(idx)
     if typ is pd.RangeIndex:
@@ -385,8 +425,10 @@ def _nonempty_scalar(x):
                         "'{0}'".format(type(x).__name__))
 
 
-def _nonempty_series(s, idx):
-
+@meta_nonempty.register(pd.Series)
+def _nonempty_series(s, idx=None):
+    if idx is None:
+        idx = _nonempty_index(s.index)
     dtype = s.dtype
     if is_datetime64tz_dtype(dtype):
         entry = pd.Timestamp('1970-01-01', tz=dtype.tz)
@@ -407,32 +449,6 @@ def _nonempty_series(s, idx):
     return pd.Series(data, name=s.name, index=idx)
 
 
-def meta_nonempty(x):
-    """Create a nonempty pandas object from the given metadata.
-
-    Returns a pandas DataFrame, Series, or Index that contains two rows
-    of fake data.
-    """
-    if isinstance(x, pd.Index):
-        return _nonempty_index(x)
-    elif isinstance(x, pd.Series):
-        idx = _nonempty_index(x.index)
-        return _nonempty_series(x, idx)
-    elif isinstance(x, pd.DataFrame):
-        idx = _nonempty_index(x.index)
-        data = {i: _nonempty_series(x.iloc[:, i], idx)
-                for i, c in enumerate(x.columns)}
-        res = pd.DataFrame(data, index=idx,
-                           columns=np.arange(len(x.columns)))
-        res.columns = x.columns
-        return res
-    elif is_scalar(x):
-        return _nonempty_scalar(x)
-    else:
-        raise TypeError("Expected Index, Series, DataFrame, or scalar, "
-                        "got {0}".format(type(x).__name__))
-
-
 def check_meta(x, meta, funcname=None, numeric_equal=True):
     """Check that the dask metadata matches the result.
 
@@ -469,14 +485,18 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
             return a == b
         return (a.kind in eq_types and b.kind in eq_types) or (a == b)
 
-    if not isinstance(meta, (pd.Series, pd.Index, pd.DataFrame)):
+    from .core import parallel_types
+    if not isinstance(meta, parallel_types()):
         raise TypeError("Expected partition to be DataFrame, Series, or "
                         "Index, got `%s`" % type(meta).__name__)
 
+    def is_dataframe_like(df):
+        return all(hasattr(df, attr) for attr in ['dtypes', 'columns', 'groupby', 'head'])
+
     if type(x) != type(meta):
         errmsg = ("Expected partition of type `%s` but got "
                   "`%s`" % (type(meta).__name__, type(x).__name__))
-    elif isinstance(meta, pd.DataFrame):
+    elif is_dataframe_like(meta):
         kwargs = dict()
         if PANDAS_VERSION >= LooseVersion('0.23.0'):
             kwargs['sort'] = True
@@ -528,8 +548,8 @@ def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
         if result is None:
             result = dsk.compute(scheduler='sync')
         if isinstance(dsk, dd.Index):
-            assert isinstance(result, pd.Index), type(result)
-            assert isinstance(dsk._meta, pd.Index), type(dsk._meta)
+            assert 'Index' in type(result).__name__, type(result)
+            # assert type(dsk._meta) == type(result), type(dsk._meta)
             if check_names:
                 assert dsk.name == result.name
                 assert dsk._meta.name == result.name
@@ -538,8 +558,8 @@ def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
             if check_dtypes:
                 assert_dask_dtypes(dsk, result)
         elif isinstance(dsk, dd.Series):
-            assert isinstance(result, pd.Series), type(result)
-            assert isinstance(dsk._meta, pd.Series), type(dsk._meta)
+            assert 'Series' in type(result).__name__, type(result)
+            assert type(dsk._meta) == type(result), type(dsk._meta)
             if check_names:
                 assert dsk.name == result.name, (dsk.name, result.name)
                 assert dsk._meta.name == result.name
@@ -548,9 +568,9 @@ def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
             _check_dask(dsk.index, check_names=check_names,
                         check_dtypes=check_dtypes, result=result.index)
         elif isinstance(dsk, dd.DataFrame):
-            assert isinstance(result, pd.DataFrame), type(result)
+            assert 'DataFrame' in type(result).__name__, type(result)
             assert isinstance(dsk.columns, pd.Index), type(dsk.columns)
-            assert isinstance(dsk._meta, pd.DataFrame), type(dsk._meta)
+            assert type(dsk._meta) == type(result), type(dsk._meta)
             if check_names:
                 tm.assert_index_equal(dsk.columns, result.columns)
                 tm.assert_index_equal(dsk._meta.columns, result.columns)
@@ -598,6 +618,10 @@ def assert_eq(a, b, check_names=True, check_dtypes=True,
     if not check_index:
         a = a.reset_index(drop=True)
         b = b.reset_index(drop=True)
+    if hasattr(a, 'to_pandas'):
+        a = a.to_pandas()
+    if hasattr(b, 'to_pandas'):
+        b = b.to_pandas()
     if isinstance(a, pd.DataFrame):
         a = _maybe_sort(a)
         b = _maybe_sort(b)
@@ -641,8 +665,12 @@ def assert_divisions(ddf):
         return
 
     def index(x):
-        return (x if isinstance(x, pd.Index)
-                else x.index.get_level_values(0))
+        if isinstance(x, pd.Index):
+            return x
+        try:
+            return x.index.get_level_values(0)
+        except AttributeError:
+            return x.index
 
     results = get_sync(ddf.dask, ddf.__dask_keys__())
     for i, df in enumerate(results[:-1]):
