@@ -16,45 +16,63 @@ ties in an intelligent way
       a
 
 For example after we finish ``a`` we can choose to run either ``b`` or ``c``
-next.  In this case we may prefer to start with ``c``, because it has other
-dependents.
-
-This is particularly important at the beginning of the computation when we
-often dump hundreds of leaf nodes onto the scheduler at once.  The order in
-which we start this computation can significantly change performance.
+next.  Making small decisions like this can greatly affect our performance,
+especially because the order in which we run tasks affects the order in which
+we can release memory, which operationally we find to have a large affect on
+many computation.  We want to run tasks in such a way that we keep only a small
+amount of data in memory at any given time.
 
 
-Breaking Ties
--------------
+Static Ordering
+---------------
 
 And so we create a total ordering over all nodes to serve as a tie breaker.  We
-represent this ordering with a dictionary.  Lower scores have higher priority.
+represent this ordering with a dictionary mapping keys to integer values.
+Lower scores have higher priority.  These scores correspond to the order in
+which a sequential scheduler would visit each node.
 
-    {'d': 0,
+    {'a': 0,
      'c': 1,
-     'a': 2,
+     'd': 2,
      'b': 3}
 
-There are several ways in which we might order our keys.  In practice we have
-found the following objectives important:
+There are several ways in which we might order our keys.  This is a nuanced
+process that has to take into account many different kinds of workflows, and
+operate efficiently in linear time.  We strongly recommend that readers look at
+the docstrings of tests in dask/tests/test_order.py.  These tests usually have
+graph types laid out very carefully to show the kinds of situations that often
+arise, and the order we would like to be determined.
 
-1.  **Depth first**:  By traversing a tree deeply before broadly we encourage
-    the completion of cohesive parts of the computation before starting new
-    parts.  This helps to reduce memory footprint.
-2.  **Lean**: We avoid tasks that have multiple dependents.  These tend to
-    start new branches of computation that we would prefer to avoid until we
-    have finished our current work.
-3.  **Heavy-first**: When deciding between two possible dependencies we choose
-    the dependency that is likely to take the longest first.  That way after it
-    finishes it has to stay in memory only a short time while waiting for its
-    co-dependent task to finish
 
-So we perform a depth first search where we choose to traverse down children
-with the following priority:
+Policy
+------
 
-1.  Number of dependents (smaller is better)
-2.  Number of total downstream dependencies (larger is better)
-3.  Name of the task itself, as a tie breaker
+Work towards *small goals* with *big steps*.
+
+1.  **Small goals**: prefer tasks whose final dependents have few dependencies.
+
+    We prefer to prioritize those tasks that help branches of computation that
+    can terminate quickly.
+
+    With more detail, we compute the total number of dependencies that each
+    task depends on (both its own dependencies, and the dependencies of its
+    dependencies, and so on), and then we choose those tasks that drive towards
+    results with a low number of total dependencies.  We choose to prioritize
+    tasks that work towards finishing shorter computations first.
+
+2.  **Big steps**: prefer tasks with many dependents
+
+    However, many tasks work towards the same final dependents.  Among those,
+    we choose those tasks with the most work left to do.  We want to finish
+    the larger portions of a sub-computation before we start on the smaller
+    ones.
+
+3.  **Name comparison**: break ties with key name
+
+    Often graphs are made with regular keynames.  When no other structural
+    difference exists between two keys, use the key name to break ties.
+    This relies on the regularity of graph constructors like dask.array to be a
+    good proxy for ordering.  This is usually a good idea and a sane default.
 """
 from __future__ import absolute_import, division, print_function
 
@@ -94,7 +112,7 @@ def order(dsk, dependencies=None):
     dependents = reverse_dict(dependencies)
 
     total_dependencies = ndependencies(dependencies, dependents)
-    total_dependents = ndependents(dependencies, dependents)
+    total_dependents, min_dependencies = ndependents(dependencies, dependents, total_dependencies)
 
     waiting = {k: set(v) for k, v in dependencies.items()}
 
@@ -102,7 +120,8 @@ def order(dsk, dependencies=None):
         return total_dependencies.get(x, 0), ReverseStrComparable(x)
 
     def dependents_key(x):
-        return (total_dependents.get(x, 0),
+        return (min_dependencies[x],
+                -total_dependents.get(x, 0),
                 StrComparable(x))
 
     result = dict()
@@ -122,6 +141,7 @@ def order(dsk, dependencies=None):
             continue
 
         deps = waiting[item]
+
         if deps:
             stack.append(item)
             seen.add(item)
@@ -137,40 +157,58 @@ def order(dsk, dependencies=None):
             waiting[dep].discard(item)
 
         deps = [d for d in dependents[item]
-                if d not in result and not (d in seen and waiting[d])]
+                if d not in result and not (d in seen and len(waiting[d]) > 1)]
         if len(deps) < 1000:
             deps = sorted(deps, key=dependents_key, reverse=True)
+
         stack.extend(deps)
 
     return result
 
 
-def ndependents(dependencies, dependents):
+def ndependents(dependencies, dependents, total_dependencies):
     """ Number of total data elements that depend on key
 
-    For each key we return the number of data that can only be run after this
+    For each key we return the number of keys that can only be run after this
     key is run.  The root nodes have value 1 while deep child nodes will have
     larger values.
 
+    We also return the minimum value of the maximum number of dependencies of
+    all final dependencies (see module-level comment for more)
+
     Examples
     --------
     >>> dsk = {'a': 1, 'b': (inc, 'a'), 'c': (inc, 'b')}
     >>> dependencies, dependents = get_deps(dsk)
 
-    >>> sorted(ndependents(dependencies, dependents).items())
+    >>> total_dependencies = ndependencies(dependencies, dependents)
+    >>> total_dependents, min_dependencies = ndependents(dependencies,
+    ...                                                  dependents,
+    ...                                                  total_dependencies)
+    >>> sorted(total_dependents.items())
     [('a', 3), ('b', 2), ('c', 1)]
+
+    Returns
+    -------
+    total_dependendents: Dict[key, int]
+    min_dependencies: Dict[key, int]
     """
     result = dict()
+    min_result = dict()
     num_needed = {k: len(v) for k, v in dependents.items()}
     current = {k for k, v in num_needed.items() if v == 0}
     while current:
         key = current.pop()
         result[key] = 1 + sum(result[parent] for parent in dependents[key])
+        try:
+            min_result[key] = min(min_result[parent] for parent in dependents[key])
+        except ValueError:
+            min_result[key] = total_dependencies[key]
         for child in dependencies[key]:
             num_needed[child] -= 1
             if num_needed[child] == 0:
                 current.add(child)
-    return result
+    return result, min_result
 
 
 def ndependencies(dependencies, dependents):
